@@ -1,4 +1,35 @@
 # -*- coding: utf-8 -*-
+
+# Copyright (c) 2013, Mahmoud Hashemi
+#
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are
+# met:
+#
+#    * Redistributions of source code must retain the above copyright
+#      notice, this list of conditions and the following disclaimer.
+#
+#    * Redistributions in binary form must reproduce the above
+#      copyright notice, this list of conditions and the following
+#      disclaimer in the documentation and/or other materials provided
+#      with the distribution.
+#
+#    * The names of the contributors may not be used to endorse or
+#      promote products derived from this software without specific
+#      prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+
 """Virtually every Python programmer has used Python for wrangling
 disk contents, and ``fileutils`` collects solutions to some of the
 most commonly-found gaps in the standard library.
@@ -163,7 +194,7 @@ class FilePerms(object):
         Here's an example that holds true on most systems:
 
         >>> import tempfile
-        >>> 'r' in FilePerms.from_path(tempfile.tempdir).user
+        >>> 'r' in FilePerms.from_path(tempfile.gettempdir()).user
         True
         """
         stat_res = os.stat(path)
@@ -223,8 +254,17 @@ else:
 
 
 def atomic_save(dest_path, **kwargs):
-    """A convenient interface to the :class:`AtomicSaver` type. See the
-    :class:`AtomicSaver` documentation for details.
+    """A convenient interface to the :class:`AtomicSaver` type. Example:
+
+    >>> try:
+    ...     with atomic_save("file.txt", text_mode=True) as fo:
+    ...         _ = fo.write('bye')
+    ...         1/0  # will error
+    ...         fo.write('bye')
+    ... except ZeroDivisionError:
+    ...     pass  # at least our file.txt didn't get overwritten
+
+    See the :class:`AtomicSaver` documentation for details.
     """
     return AtomicSaver(dest_path, **kwargs)
 
@@ -319,6 +359,8 @@ class AtomicSaver(object):
             from the previous file, or if the file did not exist, to
             respect the user's configured `umask`_, usually resulting
             in octal 0644 or 0664.
+        text_mode (bool): Whether to open the destination file in text
+            mode (i.e., ``'w'`` not ``'wb'``). Defaults to ``False`` (``wb``).
         part_file (str): Name of the temporary *part_file*. Defaults
             to *dest_path* + ``.part``. Note that this argument is
             just the filename, and not the full path of the part
@@ -358,7 +400,7 @@ class AtomicSaver(object):
         self.overwrite_part = kwargs.pop('overwrite_part', False)
         self.part_filename = kwargs.pop('part_file', None)
         self.rm_part_on_exc = kwargs.pop('rm_part_on_exc', True)
-        self.text_mode = kwargs.pop('text_mode', False)  # for windows
+        self.text_mode = kwargs.pop('text_mode', False)
         self.buffering = kwargs.pop('buffering', -1)
         if kwargs:
             raise TypeError('unexpected kwargs: %r' % (kwargs.keys(),))
@@ -451,13 +493,10 @@ class AtomicSaver(object):
         return
 
 
-_CUR_DIR = os.path.dirname(os.path.abspath(__file__))
-
-
-def iter_find_files(directory, patterns, ignored=None):
-    """Returns a generator that yields file paths under a *directory*
-    (recursively), matching *patterns* using `glob`_ syntax (e.g., ``*.txt``).
-    Also supports *ignored* patterns.
+def iter_find_files(directory, patterns, ignored=None, include_dirs=False):
+    """Returns a generator that yields file paths under a *directory*,
+    matching *patterns* using `glob`_ syntax (e.g., ``*.txt``). Also
+    supports *ignored* patterns.
 
     Args:
         directory (str): Path that serves as the root of the
@@ -466,9 +505,12 @@ def iter_find_files(directory, patterns, ignored=None):
             glob-formatted patterns to find under *directory*.
         ignored (str or list): A single pattern or list of
             glob-formatted patterns to ignore.
+        include_dirs (bool): Whether to include directories that match
+           patterns, as well. Defaults to ``False``.
 
     For example, finding Python files in the current directory:
 
+    >>> _CUR_DIR = os.path.dirname(os.path.abspath(__file__))
     >>> filenames = sorted(iter_find_files(_CUR_DIR, '*.py'))
     >>> os.path.basename(filenames[-1])
     'urlutils.py'
@@ -490,6 +532,14 @@ def iter_find_files(directory, patterns, ignored=None):
         ignored = [ignored]
     ign_re = re.compile('|'.join([fnmatch.translate(p) for p in ignored]))
     for root, dirs, files in os.walk(directory):
+        if include_dirs:
+            for basename in dirs:
+                if pats_re.match(basename):
+                    if ignored and ign_re.match(basename):
+                        continue
+                    filename = os.path.join(root, basename)
+                    yield filename
+
         for basename in files:
             if pats_re.match(basename):
                 if ignored and ign_re.match(basename):
@@ -505,6 +555,9 @@ def copy_tree(src, dst, symlinks=False, ignore=None):
     raise an exception if part of the tree already exists. It achieves
     this by using :func:`mkdir_p`.
 
+    As of Python 3.8, you may pass :func:`shutil.copytree` the
+    `dirs_exist_ok=True` flag to achieve the same effect.
+
     Args:
         src (str): Path of the source directory to copy.
         dst (str): Destination path. Existing directories accepted.
