@@ -1,4 +1,35 @@
 # -*- coding: utf-8 -*-
+
+# Copyright (c) 2013, Mahmoud Hashemi
+#
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are
+# met:
+#
+#    * Redistributions of source code must retain the above copyright
+#      notice, this list of conditions and the following disclaimer.
+#
+#    * Redistributions in binary form must reproduce the above
+#      copyright notice, this list of conditions and the following
+#      disclaimer in the documentation and/or other materials provided
+#      with the distribution.
+#
+#    * The names of the contributors may not be used to endorse or
+#      promote products derived from this software without specific
+#      prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+
 """:mod:`itertools` is full of great examples of Python generator
 usage. However, there are still some critical gaps. ``iterutils``
 fills many of those gaps with featureful, tested, and Pythonic
@@ -15,8 +46,6 @@ import math
 import time
 import codecs
 import random
-import socket
-import hashlib
 import itertools
 
 try:
@@ -41,6 +70,7 @@ except ImportError:
     # Python 3 compat
     _IS_PY3 = True
     basestring = (str, bytes)
+    unicode = str
     izip, xrange = zip, range
 
 
@@ -176,6 +206,100 @@ def split_iter(src, sep=None, maxsplit=None):
     return
 
 
+def lstrip(iterable, strip_value=None):
+    """Strips values from the beginning of an iterable. Stripped items will
+    match the value of the argument strip_value. Functionality is analigous
+    to that of the method str.lstrip. Returns a list.
+
+    >>> lstrip(['Foo', 'Bar', 'Bam'], 'Foo')
+    ['Bar', 'Bam']
+
+    """
+    return list(lstrip_iter(iterable, strip_value))
+
+
+def lstrip_iter(iterable, strip_value=None):
+    """Strips values from the beginning of an iterable. Stripped items will
+    match the value of the argument strip_value. Functionality is analigous
+    to that of the method str.lstrip. Returns a generator.
+
+    >>> list(lstrip_iter(['Foo', 'Bar', 'Bam'], 'Foo'))
+    ['Bar', 'Bam']
+
+    """
+    iterator = iter(iterable)
+    for i in iterator:
+        if i != strip_value:
+            yield i
+            break
+    for i in iterator:
+        yield i
+
+
+def rstrip(iterable, strip_value=None):
+    """Strips values from the end of an iterable. Stripped items will
+    match the value of the argument strip_value. Functionality is analigous
+    to that of the method str.rstrip. Returns a list.
+
+    >>> rstrip(['Foo', 'Bar', 'Bam'], 'Bam')
+    ['Foo', 'Bar']
+
+    """
+    return list(rstrip_iter(iterable,strip_value))
+
+
+def rstrip_iter(iterable, strip_value=None):
+    """Strips values from the end of an iterable. Stripped items will
+    match the value of the argument strip_value. Functionality is analigous
+    to that of the method str.rstrip. Returns a generator.
+
+    >>> list(rstrip_iter(['Foo', 'Bar', 'Bam'], 'Bam'))
+    ['Foo', 'Bar']
+
+    """
+    iterator = iter(iterable)
+    for i in iterator:
+        if i == strip_value:
+            cache = list()
+            cache.append(i)
+            broken = False
+            for i in iterator:
+                if i == strip_value:
+                    cache.append(i)
+                else:
+                    broken = True
+                    break
+            if not broken: # Return to caller here because the end of the
+                return     # iterator has been reached
+            for t in cache:
+                yield t
+        yield i
+
+
+def strip(iterable, strip_value=None):
+    """Strips values from the beginning and end of an iterable. Stripped items
+    will match the value of the argument strip_value. Functionality is
+    analigous to that of the method str.strip. Returns a list.
+
+    >>> strip(['Fu', 'Foo', 'Bar', 'Bam', 'Fu'], 'Fu')
+    ['Foo', 'Bar', 'Bam']
+
+    """
+    return list(strip_iter(iterable,strip_value))
+
+
+def strip_iter(iterable,strip_value=None):
+    """Strips values from the beginning and end of an iterable. Stripped items
+    will match the value of the argument strip_value. Functionality is
+    analigous to that of the method str.strip. Returns a generator.
+
+    >>> list(strip_iter(['Fu', 'Foo', 'Bar', 'Bam', 'Fu'], 'Fu'))
+    ['Foo', 'Bar', 'Bam']
+
+    """
+    return rstrip_iter(lstrip_iter(iterable,strip_value),strip_value)
+
+
 def chunked(src, size, count=None, **kw):
     """Returns a list of *count* chunks, each with *size* elements,
     generated from iterable *src*. If *src* is not evenly divisible by
@@ -201,7 +325,7 @@ def chunked(src, size, count=None, **kw):
 
 def chunked_iter(src, size, **kw):
     """Generates *size*-sized chunks from *src* iterable. Unless the
-    optional *fill* keyword argument is provided, iterables not even
+    optional *fill* keyword argument is provided, iterables not evenly
     divisible by *size* will have a final chunk that is smaller than
     *size*.
 
@@ -231,6 +355,8 @@ def chunked_iter(src, size, **kw):
     postprocess = lambda chk: chk
     if isinstance(src, basestring):
         postprocess = lambda chk, _sep=type(src)(): _sep.join(chk)
+        if _IS_PY3 and isinstance(src, bytes):
+            postprocess = lambda chk: bytes(chk)
     src_iter = iter(src)
     while True:
         cur_chunk = list(itertools.islice(src_iter, size))
@@ -471,9 +597,8 @@ def backoff_iter(start, stop, count=None, factor=2.0, jitter=False):
     return
 
 
-def bucketize(src, key=None, value_transform=None, key_filter=None):
-    """Group values in the *src* iterable by the value returned by *key*,
-    which defaults to :class:`bool`, grouping values by truthiness.
+def bucketize(src, key=bool, value_transform=None, key_filter=None):
+    """Group values in the *src* iterable by the value returned by *key*.
 
     >>> bucketize(range(5))
     {False: [0], True: [1, 2, 3, 4]}
@@ -481,6 +606,18 @@ def bucketize(src, key=None, value_transform=None, key_filter=None):
     >>> bucketize(range(5), is_odd)
     {False: [0, 2, 4], True: [1, 3]}
 
+    *key* is :class:`bool` by default, but can either be a callable or a string or a list
+    if it is a string, it is the name of the attribute on which to bucketize objects.
+
+    >>> bucketize([1+1j, 2+2j, 1, 2], key='real')
+    {1.0: [(1+1j), 1], 2.0: [(2+2j), 2]}
+
+    if *key* is a list, it contains the buckets where to put each object
+
+    >>> bucketize([1,2,365,4,98],key=[0,1,2,0,2])
+    {0: [1, 4], 1: [2], 2: [365, 98]}
+
+
     Value lists are not deduplicated:
 
     >>> bucketize([None, None, None, 'hello'])
@@ -510,24 +647,37 @@ def bucketize(src, key=None, value_transform=None, key_filter=None):
     """
     if not is_iterable(src):
         raise TypeError('expected an iterable')
-    if key is None:
-        key = bool
-    if not callable(key):
-        raise TypeError('expected callable key function')
+    elif isinstance(key, list):
+        if len(key) != len(src):
+            raise ValueError("key and src have to be the same length")
+        src = zip(key, src)
+
+    if isinstance(key, basestring):
+        key_func = lambda x: getattr(x, key, x)
+    elif callable(key):
+        key_func = key
+    elif isinstance(key, list):
+        key_func = lambda x: x[0]
+    else:
+        raise TypeError('expected key to be callable or a string or a list')
+
     if value_transform is None:
         value_transform = lambda x: x
     if not callable(value_transform):
         raise TypeError('expected callable value transform function')
+    if isinstance(key, list):
+        f = value_transform
+        value_transform=lambda x: f(x[1])
 
     ret = {}
     for val in src:
-        key_of_val = key(val)
+        key_of_val = key_func(val)
         if key_filter is None or key_filter(key_of_val):
             ret.setdefault(key_of_val, []).append(value_transform(val))
     return ret
 
 
-def partition(src, key=None):
+def partition(src, key=bool):
     """No relation to :meth:`str.partition`, ``partition`` is like
     :func:`bucketize`, but for added convenience returns a tuple of
     ``(truthy_values, falsy_values)``.
@@ -537,7 +687,8 @@ def partition(src, key=None):
     ['hi', 'bye']
 
     *key* defaults to :class:`bool`, but can be carefully overridden to
-    use any function that returns either ``True`` or ``False``.
+    use either a function that returns either ``True`` or ``False`` or
+    a string name of the attribute on which to partition objects.
 
     >>> import string
     >>> is_digit = lambda x: x in string.digits
@@ -602,10 +753,10 @@ def unique_iter(src, key=None):
 def redundant(src, key=None, groups=False):
     """The complement of :func:`unique()`.
 
-    By default returns non-unique values as a list of the *first*
-    redundant value in *src*. Pass ``groups=True`` to get groups of
-    all values with redundancies, ordered by position of the first
-    redundant value. This is useful in conjunction with some
+    By default returns non-unique/duplicate values as a list of the
+    *first* redundant value in *src*. Pass ``groups=True`` to get
+    groups of all values with redundancies, ordered by position of the
+    first redundant value. This is useful in conjunction with some
     normalizing *key* function.
 
     >>> redundant([1, 2, 3, 4])
@@ -742,7 +893,7 @@ def flatten_iter(iterable):
             yield item
 
 def flatten(iterable):
-    """``flatten_iter()`` returns a collapsed list of all the elements from
+    """``flatten()`` returns a collapsed list of all the elements from
     *iterable* while collapsing any nested iterables.
 
     >>> nested = [[1, 2], [[3], [4, 5]]]
@@ -1163,10 +1314,13 @@ class GUIDerator(object):
         self.size = size
         if size < 20 or size > 36:
             raise ValueError('expected 20 < size <= 36')
+        import hashlib
+        self._sha1 = hashlib.sha1
         self.count = itertools.count()
         self.reseed()
 
     def reseed(self):
+        import socket
         self.pid = os.getpid()
         self.salt = '-'.join([str(self.pid),
                               socket.gethostname() or b'<nohostname>',
@@ -1185,14 +1339,14 @@ class GUIDerator(object):
             if os.getpid() != self.pid:
                 self.reseed()
             target_bytes = (self.salt + str(next(self.count))).encode('utf8')
-            hash_text = hashlib.sha1(target_bytes).hexdigest()[:self.size]
+            hash_text = self._sha1(target_bytes).hexdigest()[:self.size]
             return hash_text
     else:
         def __next__(self):
             if os.getpid() != self.pid:
                 self.reseed()
-            return hashlib.sha1(self.salt +
-                                str(next(self.count))).hexdigest()[:self.size]
+            return self._sha1(self.salt +
+                              str(next(self.count))).hexdigest()[:self.size]
 
     next = __next__
 
@@ -1227,13 +1381,13 @@ class SequentialGUIDerator(GUIDerator):
     if _IS_PY3:
         def reseed(self):
             super(SequentialGUIDerator, self).reseed()
-            start_str = hashlib.sha1(self.salt.encode('utf8')).hexdigest()
+            start_str = self._sha1(self.salt.encode('utf8')).hexdigest()
             self.start = int(start_str[:self.size], 16)
             self.start |= (1 << ((self.size * 4) - 2))
     else:
         def reseed(self):
             super(SequentialGUIDerator, self).reseed()
-            start_str = hashlib.sha1(self.salt).hexdigest()
+            start_str = self._sha1(self.salt).hexdigest()
             self.start = int(start_str[:self.size], 16)
             self.start |= (1 << ((self.size * 4) - 2))
 
@@ -1294,6 +1448,47 @@ def soft_sorted(iterable, first=None, last=None, key=None, reverse=False):
         last = sorted([x for x in seq if key(x) in last], key=lambda x: last.index(key(x)))
     return first + other + last
 
+
+def untyped_sorted(iterable, key=None, reverse=False):
+    """A version of :func:`sorted` which will happily sort an iterable of
+    heterogenous types and return a new list, similar to legacy Python's
+    behavior.
+
+    >>> untyped_sorted(['abc', 2.0, 1, 2, 'def'])
+    [1, 2.0, 2, 'abc', 'def']
+
+    Note how mutually orderable types are sorted as expected, as in
+    the case of the integers and floats above.
+
+    .. note::
+
+       Results may vary across Python versions and builds, but the
+       function will produce a sorted list, except in the case of
+       explicitly unorderable objects.
+
+    """
+    class _Wrapper(object):
+        slots = ('obj',)
+
+        def __init__(self, obj):
+            self.obj = obj
+
+        def __lt__(self, other):
+            obj = key(self.obj) if key is not None else self.obj
+            other = key(other.obj) if key is not None else other.obj
+            try:
+                ret = obj < other
+            except TypeError:
+                ret = ((type(obj).__name__, id(type(obj)), obj)
+                        < (type(other).__name__, id(type(other)), other))
+            return ret
+
+    if key is not None and not callable(key):
+        raise TypeError('expected function or callable object for key, not: %r'
+                        % key)
+
+    return sorted(iterable, key=_Wrapper, reverse=reverse)
+
 """
 May actually be faster to do an isinstance check for a str path
 
