@@ -1,4 +1,35 @@
 # -*- coding: utf-8 -*-
+
+# Copyright (c) 2013, Mahmoud Hashemi
+#
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are
+# met:
+#
+#    * Redistributions of source code must retain the above copyright
+#      notice, this list of conditions and the following disclaimer.
+#
+#    * Redistributions in binary form must reproduce the above
+#      copyright notice, this list of conditions and the following
+#      disclaimer in the documentation and/or other materials provided
+#      with the distribution.
+#
+#    * The names of the contributors may not be used to endorse or
+#      promote products derived from this software without specific
+#      prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+
 """Python comes with a many great data structures, from :class:`dict`
 to :class:`collections.deque`, and no shortage of serviceable
 algorithm implementations, from :func:`sorted` to :mod:`bisect`. But
@@ -13,7 +44,7 @@ cooperative/single-threaded use cases.
 The ``queueutils`` module currently provides two Queue
 implementations: :class:`HeapPriorityQueue`, based on a heap, and
 :class:`SortedPriorityQueue`, based on a sorted list. Both use a
-unified API based on :class:`BasePriortyQueue` to facilitate testing
+unified API based on :class:`BasePriorityQueue` to facilitate testing
 the slightly different performance characteristics on various
 application use cases.
 
@@ -68,12 +99,12 @@ class BasePriorityQueue(object):
 
     Args:
         priority_key (callable): A function that takes *priority* as
-            passed in by :meth:`add` and returns an integer
+            passed in by :meth:`add` and returns a real number
             representing the effective priority.
 
     """
     # negating priority means larger numbers = higher priority
-    _default_priority_key = staticmethod(lambda p: -int(p or 0))
+    _default_priority_key = staticmethod(lambda p: -float(p or 0))
     _backend_type = list
 
     def __init__(self, **kw):
