@@ -1,4 +1,35 @@
 # -*- coding: utf-8 -*-
+
+# Copyright (c) 2013, Mahmoud Hashemi
+#
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are
+# met:
+#
+#    * Redistributions of source code must retain the above copyright
+#      notice, this list of conditions and the following disclaimer.
+#
+#    * Redistributions in binary form must reproduce the above
+#      copyright notice, this list of conditions and the following
+#      disclaimer in the documentation and/or other materials provided
+#      with the distribution.
+#
+#    * The names of the contributors may not be used to endorse or
+#      promote products derived from this software without specific
+#      prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+
 """\
 
 The :class:`set` type brings the practical expressiveness of
@@ -143,6 +174,18 @@ class IndexedSet(MutableSet):
             real_index += d_stop - d_start
         return real_index
 
+    def _get_apparent_index(self, index):
+        if index < 0:
+            index += len(self)
+        if not self.dead_indices:
+            return index
+        apparent_index = index
+        for d_start, d_stop in self.dead_indices:
+            if index < d_start:
+                break
+            apparent_index -= d_stop - d_start
+        return apparent_index
+
     def _add_dead(self, start, stop=None):
         # TODO: does not handle when the new interval subsumes
         # multiple existing intervals
@@ -293,9 +336,13 @@ class IndexedSet(MutableSet):
 
     __or__  = __ror__  = union
     __and__ = __rand__ = intersection
-    __sub__ = __rsub__ = difference
+    __sub__ = difference
     __xor__ = __rxor__ = symmetric_difference
 
+    def __rsub__(self, other):
+        vals = [x for x in other if x not in self]
+        return type(other)(vals)
+
     # in-place set operations
     def update(self, *others):
         "update(*others) -> add values from one or more iterables"
@@ -419,7 +466,7 @@ class IndexedSet(MutableSet):
     def index(self, val):
         "index(val) -> get the index of a value, raises if not present"
         try:
-            return self.item_index_map[val]
+            return self._get_apparent_index(self.item_index_map[val])
         except KeyError:
             cn = self.__class__.__name__
             raise ValueError('%r is not in %s' % (val, cn))
@@ -473,15 +520,14 @@ def complement(wrapped):
     way to invert an expression, you can just throw a complement on
     the set. Consider this example of a name filter::
 
-    >>> class NamesFilter(object):
-    ...    def __init__(self, allowed):
-    ...        self._allowed = allowed
-    ...
-    ...    def filter(self, names):
-    ...        return [name for name in names if name in self._allowed]
-
-    >>> NamesFilter(set(['alice', 'bob'])).filter(['alice', 'bob', 'carol'])
-    ['alice', 'bob']
+        >>> class NamesFilter(object):
+        ...    def __init__(self, allowed):
+        ...        self._allowed = allowed
+        ...
+        ...    def filter(self, names):
+        ...        return [name for name in names if name in self._allowed]
+        >>> NamesFilter(set(['alice', 'bob'])).filter(['alice', 'bob', 'carol'])
+        ['alice', 'bob']
 
     What if we want to just express "let all the names through"?
 
@@ -862,6 +908,22 @@ class _ComplementSet(object):
             else:  # + +
                 return _ComplementSet(included=self._included.difference(inc))
 
+    def __rsub__(self, other):
+        inc, exc = _norm_args_notimplemented(other)
+        if inc is NotImplemented:
+            return NotImplemented
+        # rsub, so the expression being evaluated is "other - self"
+        if self._included is None:
+            if exc is None:  # - +
+                return _ComplementSet(included=inc & self._excluded)
+            else:  # - -
+                return _ComplementSet(included=self._excluded - exc)
+        else:
+            if inc is None:  # + -
+                return _ComplementSet(excluded=exc | self._included)
+            else:  # + +
+                return _ComplementSet(included=inc.difference(self._included))
+
     def difference_update(self, other):
         try:
             self -= other
@@ -895,11 +957,18 @@ class _ComplementSet(object):
         return hash(self._included) ^ hash(self._excluded)
 
     def __len__(self):
-        if self._included:
+        if self._included is not None:
             return len(self._included)
         raise NotImplementedError('complemented sets have undefined length')
 
     def __iter__(self):
-        if self._included:
+        if self._included is not None:
             return iter(self._included)
         raise NotImplementedError('complemented sets have undefined contents')
+
+    def __bool__(self):
+        if self._included is not None:
+            return bool(self._included)
+        return True
+
+    __nonzero__ = __bool__  # py2 compat
