@@ -502,3 +502,36 @@ def test_seqguiderator():
     for x in range(10000):
         guid_iter = GUIDerator(size=26)
         assert len(next(guid_iter)) == 26
+
+
+def test_chunked_bytes():
+    # see #231
+    from boltons.iterutils import chunked
+
+    assert chunked(b'123', 2) in (['12', '3'], [b'12', b'3'])
+
+
+def test_lstrip():
+    from boltons.iterutils import lstrip
+
+    assert lstrip([0,1,0,2,0,3,0],0) == [1,0,2,0,3,0]
+    assert lstrip([0,0,0,1,0,2,0,3,0],0) == [1,0,2,0,3,0]
+    assert lstrip([]) == []
+
+
+
+def test_rstrip():
+    from boltons.iterutils import rstrip
+
+    assert rstrip([0,1,0,2,0,3,0],0) == [0,1,0,2,0,3]
+    assert rstrip([0,1,0,2,0,3,0,0,0],0) == [0,1,0,2,0,3]
+    assert rstrip([]) == []
+
+
+def test_strip():
+    from boltons.iterutils import strip
+
+    assert strip([0,1,0,2,0,3,0],0) == [1,0,2,0,3]
+    assert strip([0,0,0,1,0,2,0,3,0,0,0],0) == [1,0,2,0,3]
+    assert strip([]) == []
+
