@@ -28,6 +28,13 @@ def test_indexed_set_basic():
     assert x[2:4:-1] == IndexedSet([8, 7])
 
 
+def test_indexed_set_rsub():
+    "From #252"
+    assert (set('abc') - IndexedSet('bcd')) == set(['a'])
+    assert (IndexedSet('abc') - IndexedSet('bcd')) == IndexedSet(['a'])
+    assert (frozenset('abc') - IndexedSet('bcd')) == frozenset(['a'])
+
+
 def test_indexed_set_mutate():
     thou = IndexedSet(range(1000))
     assert (thou.pop(), thou.pop()) == (999, 998)
@@ -113,9 +120,12 @@ def test_complement_set():
     assert (sab ^ cab) == (cbc ^ sbc)
     assert cab - cc == sc
     assert cab - sab == cab
+    assert sab - cab == sab
     assert (cab ^ cbc | set('b')) == (sab | sbc)
     everything = complement(frozenset())
     assert everything in everything  # https://en.wikipedia.org/wiki/Russell%27s_paradox
+    assert bool(cab)
+    assert not complement(u)
     # destructive testing
     cab ^= sab
     cab ^= sab
@@ -154,3 +164,49 @@ def test_complement_set():
     with raises(TypeError): opsmash(cab, object())
     assert opsmash(ops, cab) == ops
     assert opsmash(cab, ops) == ops
+
+
+def test_iset_index_method():
+    original_list = list(range(8, 20)) + list(range(8))
+
+    indexed_list = IndexedSet()
+
+    for i in original_list:
+        indexed_list.add(i)
+
+    for i in original_list:
+        index = indexed_list.index(i)
+        # if we're removing them in order, the target value should always be at index 0
+        assert index == 0
+        indexed_list.pop(index)
+
+    indexed_list = IndexedSet(range(10))
+
+    for i in reversed(range(10)):
+        if i % 2:
+            continue
+        index = indexed_list.index(i)
+        assert i == indexed_list.pop(index)
+
+
+    indexed_list = IndexedSet(range(32))
+
+    for i in list(indexed_list):
+        if i % 3:
+            index = indexed_list.index(i)
+            assert i == indexed_list.pop(index)
+
+    indexed_list = IndexedSet(range(10))
+
+    for i in range(10):
+        if i < 3:
+            continue
+        index = indexed_list.index(i)
+        assert i == indexed_list.pop(index)
+
+    indexed_list = IndexedSet(range(32))
+
+    for i in list(indexed_list):
+        if i % 3:
+            index = indexed_list.index(i)
+            assert i == indexed_list.pop(index)
